@@ -7,18 +7,17 @@
  */
 
 #include <config.h>
+
 #include <string.h>
-#include "gal-a11y-e-text.h"
-#include "gal-a11y-util.h"
-#include <atk/atkobject.h>
-#include <atk/atktable.h>
-#include <atk/atkcomponent.h>
-#include <atk/atkobjectfactory.h>
-#include <atk/atkregistry.h>
-#include <atk/atkgobjectaccessible.h>
+
+#include <gtk/gtk.h>
+
+#include "gal/a11y/gal-a11y-util.h"
 #include "gal/e-text/e-text.h"
 #include "gal/e-text/e-text-model-repos.h"
-#include <gtk/gtkmain.h>
+
+#include "gal-a11y-e-text.h"
+#include "gal-a11y-e-text-factory.h"
 
 #define CS_CLASS(a11y) (G_TYPE_INSTANCE_GET_CLASS ((a11y), C_TYPE_STREAM, GalA11yETextClass))
 static GObjectClass *parent_class;
@@ -1119,3 +1118,14 @@ gal_a11y_e_text_get_type (void)
 
 	return type;
 }
+
+void
+gal_a11y_e_text_init (void)
+{
+	if (atk_get_root ())
+		atk_registry_set_factory_type (atk_get_default_registry (),
+					E_TYPE_TEXT,
+					gal_a11y_e_text_factory_get_type ());
+
+}
+
