@@ -35,30 +35,34 @@
  */
 
 #include <config.h>
+
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
 #include <string.h>
-#include <gdk/gdkx.h> /* for BlackPixel */
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
-#include "e-cell-text.h"
+
+#include "gal/a11y/e-table/gal-a11y-e-cell-registry.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-text.h"
+#include "gal/e-text/e-text.h"
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-text-event-processor.h"
+#include "gal/util/e-text-event-processor-emacs-like.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-unicode.h"
+
+#include "e-cell-text.h"
 #include "e-table-item.h"
-#include "gal/util/e-text-event-processor.h"
-#include "gal/e-text/e-text.h"
-#include "gal/util/e-text-event-processor-emacs-like.h"
-#include "gal/util/e-i18n.h"
 #include "e-table-tooltip.h"
-#include "gal/a11y/e-table/gal-a11y-e-cell-registry.h"
-#include "gal/a11y/e-table/gal-a11y-e-cell-text.h"
 
 #define d(x)
 #define DO_SELECTION 1
+#define VIEW_TO_CELL(view) E_CELL_TEXT (((ECellView *)view)->ecell)
 
 #if d(!)0
 #define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
@@ -94,6 +98,15 @@ enum {
 	E_SELECTION_CLIPBOARD
 };
 
+/* signals */
+enum {
+	TEXT_INSERTED,
+	TEXT_DELETED,
+	LAST_SIGNAL
+};
+
+static guint signals [LAST_SIGNAL] = { 0 };
+
 static GdkAtom clipboard_atom = GDK_NONE;
 
 #define PARENT_TYPE e_cell_get_type ()
@@ -1698,6 +1711,30 @@ e_cell_text_class_init (GObjectClass *object_class)
 
 	parent_class = g_type_class_ref (PARENT_TYPE);
 
+	signals [TEXT_INSERTED] = 
+		g_signal_new ("text_inserted",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ECellTextClass, text_inserted),
+			      NULL, NULL,
+			      e_marshal_VOID__POINTER_INT_INT_INT_INT,
+			      G_TYPE_NONE, 5,
+			      G_TYPE_POINTER, G_TYPE_INT, G_TYPE_INT,
+			      G_TYPE_INT, G_TYPE_INT);
+
+	signals [TEXT_DELETED] = 
+		g_signal_new ("text_deleted",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ECellTextClass, text_deleted),
+			      NULL, NULL,
+			      e_marshal_VOID__POINTER_INT_INT_INT_INT,
+			      G_TYPE_NONE, 5,
+			      G_TYPE_POINTER, G_TYPE_INT, G_TYPE_INT,
+			      G_TYPE_INT, G_TYPE_INT);
+
+
+
 	g_object_class_install_property (object_class, PROP_STRIKEOUT_COLUMN,
 					 g_param_spec_int ("strikeout_column",
 							   _("Strikeout Column"),
@@ -1796,15 +1833,12 @@ static gboolean
 e_cell_text_retrieve_surrounding_cb (GtkIMContext *context,
 				ECellTextView        *tv)
 {
-	int cur_pos = 0;
 	CellEdit *edit = tv->edit;
 
-	cur_pos = g_utf8_pointer_to_offset (edit->text, edit->text + edit->selection_start);
-
 	gtk_im_context_set_surrounding (context,
 					edit->text,
 					strlen (edit->text),
-					cur_pos
+					MIN (edit->selection_start, edit->selection_end)
 					);
 	
 	return TRUE;
@@ -1816,11 +1850,25 @@ e_cell_text_delete_surrounding_cb   (GtkIMContext *context,
 				gint          n_chars,
 				ECellTextView        *tv)
 {
+	int begin_pos, end_pos;
+	glong text_len;
 	CellEdit *edit = tv->edit;
 
-	gtk_editable_delete_text (GTK_EDITABLE (edit),
-				  edit->selection_end + offset,
-				  edit->selection_end + offset + n_chars);
+	text_len = g_utf8_strlen (edit->text, -1);
+	begin_pos = g_utf8_pointer_to_offset (edit->text,
+			                      edit->text + MIN (edit->selection_start, edit->selection_end));
+	begin_pos += offset;
+	end_pos = begin_pos + n_chars;
+	if(begin_pos < 0 || text_len < begin_pos)
+		return FALSE;
+	if(end_pos > text_len)
+		end_pos = text_len;
+	edit->selection_start = g_utf8_offset_to_pointer (edit->text, begin_pos)
+		                - edit->text;
+	edit->selection_end = g_utf8_offset_to_pointer (edit->text, end_pos)
+		              - edit->text;
+
+	_delete_selection (tv);
 
 	return TRUE;
 }
@@ -2182,6 +2230,8 @@ _delete_selection (ECellTextView *text_view)
 	memmove (sp, ep, length);
 
 	edit->selection_end = edit->selection_start;
+
+	g_signal_emit (VIEW_TO_CELL (text_view), signals[TEXT_DELETED], 0, text_view, edit->selection_start, ep-sp, edit->row, edit->model_col);
 }
 
 /* fixme: */
@@ -2209,6 +2259,8 @@ _insert (ECellTextView *text_view, char *string, int value)
 
 	edit->selection_start += value;
 	edit->selection_end = edit->selection_start;
+
+	g_signal_emit (VIEW_TO_CELL (text_view), signals[TEXT_INSERTED], 0, text_view, edit->selection_end-value, value, edit->row, edit->model_col);
 }
 
 static void
@@ -2778,3 +2830,39 @@ e_cell_text_delete_selection (ECellView *cell_view, gint col, gint row)
 	command.position = E_TEP_SELECTION;
 	e_cell_text_view_command (edit->tep, &command, edit);
 }
+
+/**
+ * e_cell_text_get_text_by_view:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the model
+ * @row: row of the given cell in the model
+ * 
+ * Get the cell's text directly from CellEdit,
+ * during editting this cell, the cell's text value maybe inconsistant
+ * with the text got from table_model.
+ * The caller should free the text after using it.
+ *
+ * This API is most likely to be used by a11y implementations.
+ */
+char *
+e_cell_text_get_text_by_view (ECellView *cell_view, gint col, gint row)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+	gchar	*ret, *model_text;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	
+	if (edit && ectv->edit->row == row && ectv->edit->model_col == col) { /* being editted now */
+		ret = g_strdup (edit->text);
+	} else{
+		model_text = e_cell_text_get_text (E_CELL_TEXT (cell_view->ecell), 
+					     cell_view->e_table_model, col, row);
+		ret = g_strdup (model_text);
+		e_cell_text_free_text (E_CELL_TEXT (cell_view->ecell), model_text);
+	}
+
+	return ret;
+
+}
