@@ -22,17 +22,18 @@
  */
 
 #include <config.h>
-#include <gtk/gtkenums.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtksignal.h>
+
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
-#include "e-cell-toggle.h"
+
+#include "gal/a11y/e-table/gal-a11y-e-cell-toggle.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-registry.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-hsv-utils.h"
+
+#include "e-cell-toggle.h"
 #include "e-table-item.h"
-#include "gal/a11y/e-table/gal-a11y-e-cell-toggle.h"
 
 #define PARENT_TYPE e_cell_get_type ()
 
@@ -200,7 +201,7 @@ etog_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	
 	selected = flags & E_CELL_SELECTED;
 
-	if (value >= toggle->n_states){
+	if (value < 0 || value >= toggle->n_states){
 		g_warning ("Value from the table model is %d, the states we support are [0..%d)\n",
 			   value, toggle->n_states);
 		return;
