@@ -33,24 +33,24 @@
 #include <math.h>
 #include <stdio.h>
 
-#include <gdk/gdkx.h> /* for BlackPixel */
-#include <gtk/gtkenums.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkinvisible.h>
-#include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 
-#include "e-tree-table-adapter.h"
-#include "e-tree-model.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-registry.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-tree.h"
 #include "gal/util/e-util.h"
-#include "e-table-item.h"
+
 #include "e-cell-tree.h"
+#include "e-table-item.h"
+#include "e-tree.h"
+#include "e-tree-model.h"
+#include "e-tree-table-adapter.h"
 
 #include "tree-expanded.xpm"
 #include "tree-unexpanded.xpm"
 
+
 #define PARENT_TYPE e_cell_get_type ()
 
 typedef struct {
@@ -59,6 +59,9 @@ typedef struct {
 	GdkGC       *gc;
 
 	GnomeCanvas *canvas;
+	gboolean retro_look;
+	gboolean prelit;
+	gint animate_timeout;
 
 } ECellTreeView;
 
@@ -66,6 +69,12 @@ static ECellClass *parent_class;
 
 #define INDENT_AMOUNT 16
 
+ECellView *
+e_cell_tree_view_get_subcell_view (ECellView *ect)
+{
+	return ((ECellTreeView *)ect)->subcell_view;
+}
+
 static ETreePath
 e_cell_tree_get_node (ETableModel *table_model, int row)
 {
@@ -161,7 +170,9 @@ ect_realize (ECellView *ecell_view)
 	tree_view->gc = gdk_gc_new (GTK_WIDGET (tree_view->canvas)->window);
 
 	gdk_gc_set_line_attributes (tree_view->gc, 1, 
-				    GDK_LINE_ON_OFF_DASH, None, None);
+				    GDK_LINE_ON_OFF_DASH,
+				    GDK_CAP_NOT_LAST,
+				    GDK_JOIN_MITER);
 	gdk_gc_set_dashes (tree_view->gc, 0, "\1\1", 2);
 
 	if (parent_class->realize)
@@ -186,6 +197,40 @@ ect_unrealize (ECellView *ecv)
 		(* parent_class->unrealize) (ecv);
 }
 
+static void
+draw_retro_expander (ECellTreeView *ectv, GdkDrawable *drawable, gboolean expanded, GdkRectangle *rect)
+{
+	GdkPixbuf *image;
+	int image_width, image_height;
+	ECellTree *ect = E_CELL_TREE(ectv->cell_view.ecell);
+
+	image = expanded ? ect->open_pixbuf : ect->closed_pixbuf;
+
+	image_width = gdk_pixbuf_get_width(image);
+	image_height = gdk_pixbuf_get_height(image);
+
+	gdk_pixbuf_render_to_drawable_alpha (image,
+					     drawable,
+					     rect->x, rect->y,
+					     rect->width - image_width / 2, 
+					     rect->height - image_height / 2,
+					     image_width, image_height,
+					     GDK_PIXBUF_ALPHA_BILEVEL,
+					     128,
+					     GDK_RGB_DITHER_NORMAL,
+					     image_width, 0);
+}
+
+static void
+draw_expander (ECellTreeView *ectv, GdkDrawable *drawable, GtkExpanderStyle expander_style, GtkStateType state, GdkRectangle *rect)
+{
+	GtkWidget *tree = GTK_WIDGET (ectv->canvas)->parent;
+	gint exp_size;
+	gtk_widget_style_get (tree, "expander_size", &exp_size, NULL);
+
+	gtk_paint_expander (tree->style, drawable, state, rect, tree, "treeview", rect->x + rect->width - exp_size / 2, rect->y + rect->height / 2, expander_style);
+}
+
 /*
  * ECell::draw method
  */
@@ -198,7 +243,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
 	ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
 	ETreePath node;
-	GdkRectangle rect, *clip_rect;
+	GdkRectangle rect, *clip_rect = NULL;
 	GtkWidget *canvas = GTK_WIDGET (tree_view->canvas);
 	GdkGC *fg_gc = canvas->style->fg_gc[GTK_STATE_ACTIVE];
 	GdkColor *foreground;
@@ -213,6 +258,12 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 		GdkPixbuf *node_image;
 		int node_image_width = 0, node_image_height = 0;
 		ETreePath parent_node;
+		ETree *tree = E_TREE (canvas->parent);
+
+		gtk_widget_style_get (GTK_WIDGET (tree),
+				      "retro_look", &tree_view->retro_look,
+				      NULL);
+		tree_view->prelit = FALSE;
 
 		node = e_cell_tree_get_node (ecell_view->e_table_model, row);
 
@@ -247,7 +298,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 		gdk_gc_set_foreground (tree_view->gc, foreground);
 
 		/* draw our lines */
-		if (E_CELL_TREE(tree_view->cell_view.ecell)->draw_lines) {
+		if (tree_view->retro_look && E_CELL_TREE(tree_view->cell_view.ecell)->draw_lines) {
 
 			int depth;
 
@@ -264,7 +315,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 					       rect.x + offset - INDENT_AMOUNT / 2,
 					       rect.y,
 					       rect.x + offset - INDENT_AMOUNT / 2,
-					       (e_tree_model_node_get_next (tree_model, node)
+					       (e_tree_table_adapter_node_get_next (tree_table_adapter, node)
 						? rect.y + rect.height
 						: rect.y + rect.height / 2));
 			}
@@ -276,7 +327,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 			offset -= INDENT_AMOUNT;
 			depth = visible_depth_of_node (ecell_view->e_table_model, row) - 1;
 			while (parent_node && depth != 0) {
-				if (e_tree_model_node_get_next(tree_model, parent_node)) {
+				if (e_tree_table_adapter_node_get_next(tree_table_adapter, parent_node)) {
 					gdk_draw_line (drawable, tree_view->gc,
 						       rect.x + offset - INDENT_AMOUNT / 2,
 						       rect.y,
@@ -291,26 +342,19 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 
 		/* now draw our icon if we're expandable */
 		if (e_tree_model_node_is_expandable (tree_model, node)) {
-			GdkPixbuf *image;
-			int image_width, image_height;
-
-			image = (e_tree_table_adapter_node_is_expanded (tree_table_adapter, node)
-				 ? E_CELL_TREE(tree_view->cell_view.ecell)->open_pixbuf
-				 : E_CELL_TREE(tree_view->cell_view.ecell)->closed_pixbuf);
-
-			image_width = gdk_pixbuf_get_width(image);
-			image_height = gdk_pixbuf_get_height(image);
-
-			gdk_pixbuf_render_to_drawable_alpha (image,
-							     drawable,
-							     0, 0,
-							     x1 + subcell_offset - INDENT_AMOUNT / 2 - image_width / 2,
-							     y1 + (y2 - y1) / 2 - image_height / 2,
-							     image_width, image_height,
-							     GDK_PIXBUF_ALPHA_BILEVEL,
-							     128,
-							     GDK_RGB_DITHER_NORMAL,
-							     image_width, 0);
+			gboolean expanded = e_tree_table_adapter_node_is_expanded (tree_table_adapter, node);
+			GdkRectangle r;
+			if (tree_view->retro_look) {
+				r.x = 0;
+				r.y = 0;
+				r.width = x1 + subcell_offset - INDENT_AMOUNT / 2,
+				r.height = y1 + (y2 - y1) / 2,
+				draw_retro_expander (tree_view, drawable, expanded, &r);
+			} else {
+				r = rect;
+				r.width -= node_image_width + 2;
+				draw_expander (tree_view, drawable, expanded ? GTK_EXPANDER_EXPANDED : GTK_EXPANDER_COLLAPSED, GTK_STATE_NORMAL, &r);
+			}
 		}
 
 		if (node_image) {
@@ -333,77 +377,45 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 	e_cell_draw (tree_view->subcell_view, drawable,
 		     model_col, view_col, row, flags,
 		     x1 + subcell_offset, y1, x2, y2);
+
+	if (clip_rect) {
+		gdk_gc_set_clip_rectangle (tree_view->gc, NULL);
+		gdk_gc_set_clip_rectangle (fg_gc, NULL);
+	}
 }
 
-/*
- * ECell::event method
- */
-static gint
-ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags, ECellActions *actions)
+static void
+adjust_event_position (GdkEvent *event, gint offset)
 {
-	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
-	ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
-	ETreeTableAdapter *tree_table_adapter = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
-	ETreePath node = e_cell_tree_get_node (ecell_view->e_table_model, row);
-	int offset = offset_of_node (ecell_view->e_table_model, row);
-
 	switch (event->type) {
-	case GDK_BUTTON_PRESS: {
-		/* if the event happened in our area of control (and
-                   we care about it), handle it. */
-
-		/* only activate the tree control if the click/release happens in the icon's area. */
-		if (event->button.x > (offset - INDENT_AMOUNT) && event->button.x < offset) {
-			if (e_tree_model_node_is_expandable (tree_model, node)) {
-				e_tree_table_adapter_node_set_expanded (tree_table_adapter,
-									node,
-									!e_tree_table_adapter_node_is_expanded(tree_table_adapter, node));
-				return TRUE;
-			}
-		}
-		else if (event->button.x < (offset - INDENT_AMOUNT))
-			return FALSE;
+	case GDK_BUTTON_PRESS:
+	case GDK_BUTTON_RELEASE:
+	case GDK_2BUTTON_PRESS:
+	case GDK_3BUTTON_PRESS:
+		event->button.x += offset;
+		break;
+	case GDK_MOTION_NOTIFY:
+		event->motion.x += offset;
+		break;
+	default:
+		break;
 	}
-	default: {
-		gint return_value;
+}
 
-		/* modify the event and pass it off to our subcell_view */
-		switch (event->type) {
-		case GDK_BUTTON_PRESS:
-		case GDK_BUTTON_RELEASE:
-		case GDK_2BUTTON_PRESS:
-		case GDK_3BUTTON_PRESS:
-			event->button.x -= offset;
-			break;
-		case GDK_MOTION_NOTIFY:
-			event->motion.x -= offset;
-			break;
-		default:
-			/* nada */
-			break;
-		}
-
-		return_value = e_cell_event(tree_view->subcell_view, event, model_col, view_col, row, flags, actions);
-
-		/* modify the event and pass it off to our subcell_view */
-		switch (event->type) {
-		case GDK_BUTTON_PRESS:
-		case GDK_BUTTON_RELEASE:
-		case GDK_2BUTTON_PRESS:
-		case GDK_3BUTTON_PRESS:
-			event->button.x += offset;
-			break;
-		case GDK_MOTION_NOTIFY:
-			event->motion.x += offset;
-			break;
-		default:
-			/* nada */
-			break;
-		}
-
-		return return_value;
-	}
+static gboolean
+event_in_expander (GdkEvent *event, gint offset, gint height)
+{
+	switch (event->type) {
+	case GDK_BUTTON_PRESS:
+		return (event->button.x > (offset - INDENT_AMOUNT) && event->button.x < offset);
+	case GDK_MOTION_NOTIFY:
+		return (event->motion.x > (offset - INDENT_AMOUNT) && event->motion.x < offset &&
+			event->motion.y > 2 && event->motion.y < (height - 2));
+	default:
+		break;
 	}
+
+	return FALSE;
 }
 
 /*
@@ -417,6 +429,128 @@ ect_height (ECellView *ecell_view, int model_col, int view_col, int row)
 	return (((e_cell_height (tree_view->subcell_view, model_col, view_col, row)) + 1) / 2) * 2;
 }
 
+typedef struct {
+	ECellTreeView *ectv;
+	ETreeTableAdapter *etta;
+	ETreePath node;
+	gboolean expanded;
+	gboolean finish;
+	GdkRectangle area;
+} animate_closure_t;
+
+static gboolean
+animate_expander (gpointer data)
+{
+	animate_closure_t *closure = (animate_closure_t *) data;
+
+	if (closure->finish) {
+		e_tree_table_adapter_node_set_expanded (closure->etta, closure->node, !closure->expanded);
+		closure->ectv->animate_timeout = 0;
+		g_free (data);
+		return FALSE;
+	}
+
+	draw_expander (closure->ectv, GTK_LAYOUT (closure->ectv->canvas)->bin_window,
+		       closure->expanded ? GTK_EXPANDER_SEMI_COLLAPSED : GTK_EXPANDER_SEMI_EXPANDED,
+		       GTK_STATE_NORMAL, &closure->area);
+	closure->finish = TRUE;
+
+	return TRUE;
+}
+
+/*
+ * ECell::event method
+ */
+static gint
+ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags, ECellActions *actions)
+{
+	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
+	ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
+	ETreeTableAdapter *etta = e_cell_tree_get_tree_table_adapter(ecell_view->e_table_model, row);
+	ETreePath node = e_cell_tree_get_node (ecell_view->e_table_model, row);
+	int offset = offset_of_node (ecell_view->e_table_model, row);
+	gint result;
+
+	switch (event->type) {
+	case GDK_BUTTON_PRESS:
+
+		if (event_in_expander (event, offset, 0)) {
+			if (e_tree_model_node_is_expandable (tree_model, node)) {
+				gboolean expanded = e_tree_table_adapter_node_is_expanded(etta, node);
+				if (tree_view->retro_look)
+					e_tree_table_adapter_node_set_expanded (etta, node, !expanded);
+				else {
+					gint tmp_row = row;
+					GdkRectangle area;
+					animate_closure_t *closure = g_new0 (animate_closure_t, 1);
+					e_table_item_get_cell_geometry (tree_view->cell_view.e_table_item_view,
+								&tmp_row, &view_col, &area.x, &area.y, NULL, &area.height);
+					area.width = offset - 2;
+					draw_expander (tree_view, GTK_LAYOUT (tree_view->canvas)->bin_window,
+						       expanded ? GTK_EXPANDER_SEMI_EXPANDED : GTK_EXPANDER_SEMI_COLLAPSED,
+						       GTK_STATE_NORMAL, &area);
+					closure->ectv = tree_view;
+					closure->etta = etta;
+					closure->node = node;
+					closure->expanded = expanded;
+					closure->area = area;
+					tree_view->animate_timeout = g_timeout_add (50, animate_expander, closure);
+				}
+				return TRUE;
+			}
+		}
+		else if (event->button.x < (offset - INDENT_AMOUNT))
+			return FALSE;
+		break;
+
+	case GDK_MOTION_NOTIFY:
+
+		if (!tree_view->retro_look && e_tree_model_node_is_expandable (tree_model, node)) {
+			gint height = ect_height (ecell_view, model_col, view_col, row);
+			GdkRectangle area;
+			gboolean in_expander = event_in_expander (event, offset, height);
+
+			if (tree_view->prelit ^ in_expander) {
+				gint tmp_row = row;
+				e_table_item_get_cell_geometry (tree_view->cell_view.e_table_item_view,
+								&tmp_row, &view_col, &area.x, &area.y, NULL, &area.height);
+				area.width = offset - 2;
+				draw_expander (tree_view, GTK_LAYOUT (tree_view->canvas)->bin_window,
+					       e_tree_table_adapter_node_is_expanded (etta, node) ? GTK_EXPANDER_EXPANDED : GTK_EXPANDER_COLLAPSED,
+					       in_expander ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL, &area);
+				tree_view->prelit = in_expander;
+				return TRUE;
+			}
+
+		}
+		break;
+
+	case GDK_LEAVE_NOTIFY:
+
+		if (tree_view->prelit) {
+			gint tmp_row = row;
+			GdkRectangle area;
+			e_table_item_get_cell_geometry (tree_view->cell_view.e_table_item_view,
+							&tmp_row, &view_col, &area.x, &area.y, NULL, &area.height);
+			area.width = offset - 2;
+			draw_expander (tree_view, GTK_LAYOUT (tree_view->canvas)->bin_window,
+				       e_tree_table_adapter_node_is_expanded (etta, node) ? GTK_EXPANDER_EXPANDED : GTK_EXPANDER_COLLAPSED,
+				       GTK_STATE_NORMAL, &area);
+			tree_view->prelit = FALSE;
+		}
+		return TRUE;
+
+	default:
+		break;
+	}
+
+	adjust_event_position (event, -offset);
+	result = e_cell_event(tree_view->subcell_view, event, model_col, view_col, row, flags, actions);
+	adjust_event_position (event, offset);
+
+	return result;
+}
+
 /*
  * ECell::max_width method
  */
@@ -434,7 +568,7 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
 	number_of_rows = e_table_model_row_count (ecell_view->e_table_model);
 
 	if (!per_row)
-	subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
+		subcell_max_width = e_cell_max_width (tree_view->subcell_view, model_col, view_col);
 	
 	for (row = 0; row < number_of_rows; row++) {
 		ETreeModel *tree_model = e_cell_tree_get_tree_model(ecell_view->e_table_model, row);
@@ -512,7 +646,7 @@ ect_show_tooltip (ECellView *ecell_view, int model_col, int view_col, int row,
 	tooltip->x += offset;
 	e_cell_show_tooltip (tree_view->subcell_view, model_col, view_col, row, col_width - offset, tooltip);
 }
-		
+
 /*
  * ECellView::get_bg_color method
  */
@@ -585,7 +719,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 						    height);
 				gnome_print_lineto (context,
 						    offset - INDENT_AMOUNT / 2,
-						    (e_tree_model_node_get_next (tree_model, node)
+						    (e_tree_table_adapter_node_get_next (tree_table_adapter, node)
 						     ? 0
 						     : height / 2));
 			}
@@ -597,7 +731,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 			depth = visible_depth_of_node (ecell_view->e_table_model, row) - 1;
 			offset -= INDENT_AMOUNT;
 			while (node && depth != 0) {
-				if (e_tree_model_node_get_next(tree_model, node)) {
+				if (e_tree_table_adapter_node_get_next(tree_table_adapter, node)) {
 					gnome_print_moveto (context,
 							    offset - INDENT_AMOUNT / 2,
 							    height);
@@ -655,16 +789,16 @@ ect_print_height (ECellView *ecell_view, GnomePrintContext *context,
 }
 
 /*
- * GtkObject::destroy method
+ * GObject::dispose method
  */
 static void
-ect_destroy (GtkObject *object)
+ect_dispose (GObject *object)
 {
 	ECellTree *ect = E_CELL_TREE (object);
 
 	/* destroy our subcell */
 	if (ect->subcell)
-		gtk_object_unref (GTK_OBJECT (ect->subcell));
+		g_object_unref (ect->subcell);
 	ect->subcell = NULL;
 
 	if (ect->open_pixbuf)
@@ -675,32 +809,34 @@ ect_destroy (GtkObject *object)
 		gdk_pixbuf_unref (ect->closed_pixbuf);
 	ect->closed_pixbuf = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
-e_cell_tree_class_init (GtkObjectClass *object_class)
+e_cell_tree_class_init (GObjectClass *object_class)
 {
 	ECellClass *ecc = (ECellClass *) object_class;
 
-	object_class->destroy = ect_destroy;
+	object_class->dispose = ect_dispose;
 
-	ecc->new_view   = ect_new_view;
-	ecc->kill_view  = ect_kill_view;
-	ecc->realize    = ect_realize;
-	ecc->unrealize  = ect_unrealize;
-	ecc->draw       = ect_draw;
-	ecc->event      = ect_event;
-	ecc->height     = ect_height;
-	ecc->enter_edit = ect_enter_edit;
-	ecc->leave_edit = ect_leave_edit;
-	ecc->print      = ect_print;
-	ecc->print_height = ect_print_height;
-	ecc->max_width = ect_max_width;
-	ecc->show_tooltip = ect_show_tooltip;
+	ecc->new_view         = ect_new_view;
+	ecc->kill_view        = ect_kill_view;
+	ecc->realize          = ect_realize;
+	ecc->unrealize        = ect_unrealize;
+	ecc->draw             = ect_draw;
+	ecc->event            = ect_event;
+	ecc->height           = ect_height;
+	ecc->enter_edit       = ect_enter_edit;
+	ecc->leave_edit       = ect_leave_edit;
+	ecc->print            = ect_print;
+	ecc->print_height     = ect_print_height;
+	ecc->max_width        = ect_max_width;
+	ecc->show_tooltip     = ect_show_tooltip;
 	ecc->get_bg_color     = ect_get_bg_color;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
+
+        gal_a11y_e_cell_registry_add_cell_type (NULL, E_CELL_TREE_TYPE, gal_a11y_e_cell_tree_new);
 }
 
 E_MAKE_TYPE(e_cell_tree, "ECellTree", ECellTree, e_cell_tree_class_init, NULL, PARENT_TYPE)
@@ -726,7 +862,7 @@ e_cell_tree_construct (ECellTree *ect,
 {		       
 	ect->subcell = subcell;
 	if (subcell) {
-		gtk_object_ref (GTK_OBJECT (subcell));
+		g_object_ref (subcell);
 		gtk_object_sink (GTK_OBJECT (subcell));
 	}
 	if (open_pixbuf)
@@ -764,9 +900,10 @@ e_cell_tree_new (GdkPixbuf *open_pixbuf,
 		 gboolean draw_lines,
 		 ECell *subcell)
 {
-	ECellTree *ect = gtk_type_new (e_cell_tree_get_type ());
+	ECellTree *ect = g_object_new (E_CELL_TREE_TYPE, NULL);
 
 	e_cell_tree_construct (ect, open_pixbuf, closed_pixbuf, draw_lines, subcell);
 
 	return (ECell *) ect;
 }
+
