@@ -29,24 +29,22 @@
 
 #include <config.h>
 
-#include "e-table-config.h"
-
 #include <stdlib.h>
 #include <string.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktogglebutton.h>
+
+#include <gtk/gtk.h>
 #include <libgnomeui/gnome-propertybox.h>
 #include <glade/glade.h>
-#include "gal/util/e-util.h"
+
 #include "gal/util/e-i18n.h"
+#include "gal/util/e-util-private.h"
+#include "gal/util/e-util.h"
 #include "gal/widgets/e-unicode.h"
 
-#include <e-table-scrolled.h>
-#include <e-table-without.h>
-#include <e-table-memory-store.h>
-
+#include "e-table-config.h"
+#include "e-table-memory-store.h"
+#include "e-table-scrolled.h"
+#include "e-table-without.h"
 
 static GObjectClass *config_parent_class;
 
@@ -947,6 +945,11 @@ config_button_up (GtkWidget *widget, ETableConfig *config)
 	int i;
 
 	e_table_selected_row_foreach (config->shown, add_column, &columns);
+
+	/* if no columns left, just return */
+	if (columns == NULL)
+		return;
+
 	columns = g_list_reverse (columns);
 
 	new_shown = g_new (int, config->temp_state->col_count);
@@ -998,6 +1001,11 @@ config_button_down (GtkWidget *widget, ETableConfig *config)
 
 	e_table_selected_row_foreach (config->shown, add_column, &columns);
 
+	/* if no columns left, just return */
+	if (columns == NULL)
+		return;
+
+
 	new_shown = g_new (int, config->temp_state->col_count);
 	new_expansions = g_new (double, config->temp_state->col_count);
 
@@ -1069,9 +1077,19 @@ setup_gui (ETableConfig *config)
 	create_global_store (config);
 
 	if (e_table_sort_info_get_can_group (config->state->sort_info)) {
-		gui = glade_xml_new (ETABLE_GLADEDIR "/e-table-config.glade", NULL, E_I18N_DOMAIN);
+		gchar *filename =
+			g_build_filename (GAL_GLADEDIR,
+					  "e-table-config.glade",
+					  NULL);
+		gui = glade_xml_new (filename, NULL, E_I18N_DOMAIN);
+		g_free (filename);
 	} else {
-		gui = glade_xml_new (ETABLE_GLADEDIR "/e-table-config-no-group.glade", NULL, E_I18N_DOMAIN);
+		gchar *filename =
+			g_build_filename (GAL_GLADEDIR,
+					  "e-table-config-no-group.glade",
+					  NULL);
+		gui = glade_xml_new (filename, NULL, E_I18N_DOMAIN);
+		g_free (filename);
 	}
 
 	g_object_unref (global_store);
@@ -1156,16 +1174,8 @@ e_table_config_construct (ETableConfig        *config,
 
 	setup_gui (config);
 
-	if (parent_window) {
-		gtk_window_set_transient_for (GTK_WINDOW (config->dialog_toplevel),
-					      parent_window);
-		/*
-		  center on parent doesn't seem to work for me, so let use GTK_WIN_POS_CENTER_ALWAYS - that is
-		  what glade used before as well, so it's not worse then it was
-		  gtk_window_set_position (GTK_WINDOW (config->dialog_toplevel), GTK_WIN_POS_CENTER_ON_PARENT);
-		*/
-		gtk_window_set_position (GTK_WINDOW (config->dialog_toplevel), GTK_WIN_POS_CENTER_ALWAYS);
-	}
+	gtk_window_set_transient_for (GTK_WINDOW (config->dialog_toplevel),
+				      parent_window);
 
 	config_sort_info_update   (config);
 	config_group_info_update  (config);
