@@ -22,41 +22,36 @@
  * 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <gdk/gdk.h>
+
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
 
+#include "gal/a11y/e-table/gal-a11y-e-table.h"
 #include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-canvas-background.h"
 #include "gal/widgets/e-canvas-vbox.h"
 #include "gal/widgets/e-unicode.h"
+
 #include "e-table.h"
+#include "e-table-click-to-add.h"
+#include "e-table-column-specification.h"
+#include "e-table-group-leaf.h"
 #include "e-table-header-item.h"
 #include "e-table-header-utils.h"
 #include "e-table-subset.h"
-#include "e-table-item.h"
-#include "e-table-group.h"
-#include "e-table-group-leaf.h"
-#include "e-table-click-to-add.h"
-#include "e-table-specification.h"
-#include "e-table-state.h"
-#include "e-table-column-specification.h"
-
 #include "e-table-utils.h"
 
-#include <atk/atk.h>
-#include "gal/a11y/e-table/gal-a11y-e-table-factory.h"
 
 #define COLUMN_HEADER_HEIGHT 16
 
@@ -1124,6 +1119,18 @@ table_canvas_focus_event_cb (GtkWidget *widget, GdkEventFocus *event, gpointer d
 		gnome_canvas_item_grab_focus (etable->click_to_add);
 	} else if (!canvas->focused_item && etable->group) {
         	focus_first_etable_item (etable->group);
+	} else if (canvas->focused_item) {
+		ESelectionModel *selection = (ESelectionModel *)etable->selection;
+
+		/* check whether click_to_add already got the focus */
+		if (etable->click_to_add) {
+			GnomeCanvasItem *row = E_TABLE_CLICK_TO_ADD(etable->click_to_add)->row;
+			if (canvas->focused_item == row)
+				return TRUE;
+		}
+
+		if (e_selection_model_cursor_row (selection) == -1)
+			focus_first_etable_item (etable->group);
 	}
 
 	return TRUE;
@@ -1514,7 +1521,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 
 	e_table->use_click_to_add = specification->click_to_add;
 	e_table->use_click_to_add_end = specification->click_to_add_end;
-	e_table->click_to_add_message = g_strdup (dgettext (e_table->domain, specification->click_to_add_message));
+	e_table->click_to_add_message = specification->click_to_add_message ? g_strdup (dgettext (e_table->domain, specification->click_to_add_message)) : NULL;
 	e_table->alternating_row_colors = specification->alternating_row_colors;
 	e_table->horizontal_draw_grid = specification->horizontal_draw_grid;
 	e_table->vertical_draw_grid = specification->vertical_draw_grid;
@@ -1934,7 +1941,15 @@ e_table_load_specification (ETable *e_table, gchar *filename)
 	g_return_val_if_fail(filename != NULL, -1);
 
 	/* doesn't work yet, yay */
+#ifdef G_OS_WIN32
+	{
+		gchar *locale_filename = gnome_win32_locale_filename_from_utf8 (filename);
+		xmlSpec = xmlParseFile (locale_filename);
+		g_free (locale_filename);
+	}
+#else
 	xmlSpec = xmlParseFile (filename);
+#endif
 	ret = et_real_set_specification(e_table, xmlSpec);
 	xmlFreeDoc (xmlSpec);
 
@@ -2388,11 +2403,13 @@ e_table_get_cell_geometry (ETable *table,
 
 	e_table_group_get_cell_geometry(table->group, &row, &col, x_return, y_return, width_return, height_return);
 
-	if (x_return)
+	if (x_return && table->table_canvas)
 		(*x_return) -= GTK_LAYOUT(table->table_canvas)->hadjustment->value;
 	if (y_return) {
-		(*y_return) -= GTK_LAYOUT(table->table_canvas)->vadjustment->value;
-		(*y_return) += GTK_WIDGET(table->header_canvas)->allocation.height;
+		if (table->table_canvas)
+			(*y_return) -= GTK_LAYOUT(table->table_canvas)->vadjustment->value;
+		if (table->header_canvas)
+			(*y_return) += GTK_WIDGET(table->header_canvas)->allocation.height;
 	}
 }
 
@@ -3329,10 +3346,7 @@ e_table_class_init (ETableClass *class)
 							      E_TABLE_MODEL_TYPE,
 							      G_PARAM_READABLE));
 
-	atk_registry_set_factory_type (atk_get_default_registry (),
-				       E_TABLE_TYPE,
-				       gal_a11y_e_table_factory_get_type ());
-
+	gal_a11y_e_table_init ();
 }
 
 E_MAKE_TYPE(e_table, "ETable", ETable, e_table_class_init, e_table_init, PARENT_TYPE)
