@@ -22,20 +22,21 @@
  * 02111-1307, USA.
  */
 
-
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdlib.h>
 #include <string.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
+
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
-#include "e-tree-table-adapter.h"
+
 #include "e-table-sorting-utils.h"
+#include "e-tree-table-adapter.h"
 
 #define PARENT_TYPE E_TABLE_MODEL_TYPE
 #define d(x)
@@ -925,7 +926,19 @@ open_file (ETreeTableAdapter *etta, const char *filename)
 	int vers;
 	gboolean model_default, saved_default;
 
+	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+		return NULL;
+
+#ifdef G_OS_WIN32
+	{
+		gchar *locale_filename = gnome_win32_locale_filename_from_utf8 (filename);
+		doc = xmlParseFile (locale_filename);
+		g_free (locale_filename);
+	}
+#else
 	doc = xmlParseFile (filename);
+#endif
+
 	if (!doc)
 		return NULL;
 
