@@ -23,24 +23,27 @@
  */
 
 #include <config.h>
+
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtktypebuiltins.h>
+#include <gtk/gtk.h>
 #include <libxml/parser.h>
 #include <libgnomecanvas/gnome-canvas.h>
-#include "gal/util/e-util.h"
+
 #include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-canvas-utils.h"
+
 #include "e-completion-view.h"
-#include "e-text.h"
 #include "e-entry.h"
+#include "e-text.h"
 
 #define MIN_ENTRY_WIDTH  150
 #define INNER_BORDER 2
@@ -503,9 +506,14 @@ e_entry_show_popup (EEntry *entry, gboolean visible)
 	if (pop == NULL)
 		return;
 
-	/* The async query can give us a result after the focus was lost by the
-	   widget.  In that case, we don't want to show the pop-up.   */
-	if (! GTK_WIDGET_HAS_FOCUS (entry->canvas))
+	/* The async query can give us a result after the focus was
+	   lost by the widget.  In that case, we don't want to show
+	   the pop-up.
+
+	   but only return early if we're popping *up* the completion
+	   view.  If we're trying to pop down the view, continue on
+	   regardless of whether or not the canvas has focus. */
+	if (/*visible && */! GTK_WIDGET_HAS_FOCUS (entry->canvas))
 		return;
 
 	if (visible) {
