@@ -22,9 +22,13 @@
  */
 
 #include <config.h>
+
 #include <ctype.h>
-#include <gtk/gtksignal.h>
-#include <gal/util/e-util.h>
+
+#include <gtk/gtk.h>
+
+#include "gal/util/e-util.h"
+
 #include "e-table-text-model.h"
 
 static void e_table_text_model_class_init (ETableTextModelClass *class);
