@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * gal-define-views-model.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -22,37 +22,43 @@
  */
 
 #include <config.h>
-#include <gnome-xml/tree.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
+
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+
 #include "gal-define-views-model.h"
 
-#define PARENT_TYPE e_table_model_get_type()
+#define PARENT_TYPE E_TABLE_MODEL_TYPE
 static ETableModelClass *parent_class;
 
 /*
  * GalDefineViewsModel callbacks
  * These are the callbacks that define the behavior of our custom model.
  */
-static void gal_define_views_model_set_arg (GtkObject *o, GtkArg *arg, guint arg_id);
-static void gal_define_views_model_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-
+static void gal_define_views_model_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void gal_define_views_model_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
 enum {
-	ARG_0,
-	ARG_EDITABLE,
-	ARG_COLLECTION
+	PROP_0,
+	PROP_EDITABLE,
+	PROP_COLLECTION
 };
 
 static void
-gdvm_destroy(GtkObject *object)
+gdvm_dispose(GObject *object)
 {
 	GalDefineViewsModel *model = GAL_DEFINE_VIEWS_MODEL(object);
 
-	gtk_object_unref(GTK_OBJECT(model->collection));
+	if (model->collection)
+		g_object_unref(model->collection);
+	model->collection = NULL;
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 /* This function returns the number of columns in our ETableModel. */
@@ -160,20 +166,29 @@ gal_define_views_model_append (GalDefineViewsModel *model,
 }
 
 static void
-gal_define_views_model_class_init (GtkObjectClass *object_class)
+gal_define_views_model_class_init (GObjectClass *object_class)
 {
 	ETableModelClass *model_class = (ETableModelClass *) object_class;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
-	object_class->destroy = gdvm_destroy;
-	object_class->set_arg   = gal_define_views_model_set_arg;
-	object_class->get_arg   = gal_define_views_model_get_arg;
+	object_class->dispose        = gdvm_dispose;
+	object_class->set_property   = gal_define_views_model_set_property;
+	object_class->get_property   = gal_define_views_model_get_property;
 
-	gtk_object_add_arg_type ("GalDefineViewsModel::editable", GTK_TYPE_BOOL,
-				 GTK_ARG_READWRITE, ARG_EDITABLE);
-	gtk_object_add_arg_type ("GalDefineViewsModel::collection", GAL_VIEW_COLLECTION_TYPE,
-				 GTK_ARG_READWRITE, ARG_COLLECTION);
+	g_object_class_install_property (object_class, PROP_EDITABLE, 
+					 g_param_spec_boolean ("editable",
+							       _("Editable"),
+							       /*_( */"XXX blurb" /*)*/,
+							       FALSE,
+							       G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_COLLECTION, 
+					 g_param_spec_object ("collection",
+							      _("Collection"),
+							      /*_( */"XXX blurb" /*)*/,
+							      GAL_VIEW_COLLECTION_TYPE,
+							      G_PARAM_READWRITE));
 
 	model_class->column_count     = gdvm_col_count;
 	model_class->row_count        = gdvm_row_count;
@@ -189,7 +204,7 @@ gal_define_views_model_class_init (GtkObjectClass *object_class)
 }
 
 static void
-gal_define_views_model_init (GtkObject *object)
+gal_define_views_model_init (GObject *object)
 {
 	GalDefineViewsModel *model = GAL_DEFINE_VIEWS_MODEL(object);
 
@@ -197,75 +212,50 @@ gal_define_views_model_init (GtkObject *object)
 }
 
 static void
-gal_define_views_model_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
-{
-	GalDefineViewsModel *model;
-
-	model = GAL_DEFINE_VIEWS_MODEL (o);
-
-	switch (arg_id){
-	case ARG_EDITABLE:
-		model->editable = GTK_VALUE_BOOL (*arg);
-		break;
-
-	case ARG_COLLECTION:
-		e_table_model_pre_change(E_TABLE_MODEL(o));
-		if (GTK_VALUE_OBJECT (*arg))
-			model->collection = GAL_VIEW_COLLECTION(GTK_VALUE_OBJECT (*arg));
-		else
-			model->collection = NULL;
-		e_table_model_changed(E_TABLE_MODEL(o));
-		break;
-	}
-}
-
-static void
-gal_define_views_model_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+gal_define_views_model_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	GalDefineViewsModel *model;
 
 	model = GAL_DEFINE_VIEWS_MODEL (object);
 
-	switch (arg_id) {
-	case ARG_EDITABLE:
-		GTK_VALUE_BOOL (*arg) = model->editable;
+	switch (prop_id){
+	case PROP_EDITABLE:
+		model->editable = g_value_get_boolean (value);
 		break;
 
-	case ARG_COLLECTION:
-		if (model->collection)
-			GTK_VALUE_OBJECT (*arg) = GTK_OBJECT(model->collection);
+	case PROP_COLLECTION:
+		e_table_model_pre_change(E_TABLE_MODEL(object));
+		if (g_value_get_object (value))
+			model->collection = GAL_VIEW_COLLECTION(g_value_get_object (value));
 		else
-			GTK_VALUE_OBJECT (*arg) = NULL;
+			model->collection = NULL;
+		e_table_model_changed(E_TABLE_MODEL(object));
 		break;
+	}
+}
+
+static void
+gal_define_views_model_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	GalDefineViewsModel *model;
+
+	model = GAL_DEFINE_VIEWS_MODEL (object);
+
+	switch (prop_id) {
+	case PROP_EDITABLE:
+		g_value_set_boolean (value, model->editable);
+		break;
+
+	case PROP_COLLECTION:
+		g_value_set_object (value, model->collection);
 
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
-GtkType
-gal_define_views_model_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"GalDefineViewsModel",
-			sizeof (GalDefineViewsModel),
-			sizeof (GalDefineViewsModelClass),
-			(GtkClassInitFunc) gal_define_views_model_class_init,
-			(GtkObjectInitFunc) gal_define_views_model_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE(gal_define_views_model, "GalDefineViewsModel", GalDefineViewsModel, gal_define_views_model_class_init, gal_define_views_model_init, PARENT_TYPE)
 
 /**
  * gal_define_views_model_new
@@ -280,7 +270,7 @@ gal_define_views_model_new (void)
 {
 	GalDefineViewsModel *et;
 
-	et = gtk_type_new (gal_define_views_model_get_type ());
+	et = g_object_new (GAL_DEFINE_VIEWS_MODEL_TYPE, NULL);
 
 	return E_TABLE_MODEL(et);
 }
