@@ -28,19 +28,17 @@
 
 #include <config.h>
 
-#include "e-icon-bar.h"
-
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtksignal.h>
-#include "e-icon-bar-bg-item.h"
-#include <gal/util/e-util.h>
-#include <gal/e-text/e-text.h>
-#include <gal/widgets/e-canvas-utils.h>
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
 
+#include "gal/e-text/e-text.h"
+#include "gal/util/e-util.h"
+#include "gal/widgets/e-canvas-utils.h"
+
+#include "e-icon-bar.h"
+#include "e-icon-bar-bg-item.h"
+
 /* These are the offsets of the icons & text in both views. Note that the
    shadow around icons is drawn in the space between items (as is the
    horizontal bar when dragging). */
