@@ -33,15 +33,15 @@
  */
 
 #include <config.h>
+
 #include <string.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkrc.h>
-#include <gtk/gtkselection.h>
-#include <gtk/gtksignal.h>
+
+#include <gtk/gtk.h>
 #include <libgnome/gnome-util.h>
-#include <gal/util/e-util.h>
-#include <gal/e-text/e-entry.h>
+
+#include "gal/e-text/e-entry.h"
+#include "gal/util/e-util.h"
+
 #include "e-shortcut-bar.h"
 #include "e-vscrolled-bar.h"
 
@@ -554,7 +554,7 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 	g_signal_connect (group->icon_bar, "drag_data_get",
 			  G_CALLBACK (e_shortcut_bar_on_drag_data_get),
 			  shortcut_bar);
-	g_signal_connect (group->icon_bar, "drag_motion",
+	g_signal_connect_after (group->icon_bar, "drag_motion",
 			  G_CALLBACK (e_shortcut_bar_on_drag_motion),
 			  shortcut_bar);
 	g_signal_connect (group->icon_bar, "drag_drop",
@@ -828,7 +828,6 @@ e_shortcut_bar_on_drag_motion (GtkWidget          *widget,
 	group_num = e_group_bar_get_group_num (E_GROUP_BAR (shortcut_bar),
 					       GTK_WIDGET (icon_bar)->parent);
 
-
 	/* We only care about the current group.  */
 	if (group_num != E_GROUP_BAR (shortcut_bar)->current_group_num) {
 		gdk_drag_status (context, 0, time);
