@@ -28,15 +28,16 @@
  */
 
 #include <config.h>
+
 #include <string.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtktable.h>
+
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
-#include <gdk/gdkcolor.h>
+
 #include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
+
 #include "color-group.h"
 #include "color-palette.h"
 #include "e-colors.h"
