@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-categories-master-list.c - The base class for the master list of categories.
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -22,17 +22,16 @@
  */
 
 #include <config.h>
-#include <gtk/gtksignal.h>
+
 #include "gal/util/e-util.h"
+
 #include "e-categories-master-list.h"
 
-#define ECML_CLASS(e) ((ECategoriesMasterListClass *)((GtkObject *)e)->klass)
-
-#define PARENT_TYPE gtk_object_get_type ()
+#define PARENT_TYPE G_TYPE_OBJECT
 
 #define d(x)
 
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class;
 
 enum {
 	CHANGED,
@@ -53,7 +52,7 @@ e_categories_master_list_count (ECategoriesMasterList *ecml)
 	g_return_val_if_fail (ecml != NULL, 0);
 	g_return_val_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml), 0);
 
-	return ECML_CLASS (ecml)->count (ecml);
+	return E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->count (ecml);
 }
 
 /**
@@ -69,7 +68,7 @@ e_categories_master_list_nth (ECategoriesMasterList *ecml, int n)
 	g_return_val_if_fail (ecml != NULL, NULL);
 	g_return_val_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml), NULL);
 
-	return ECML_CLASS (ecml)->nth (ecml, n);
+	return E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->nth (ecml, n);
 }
 
 /**
@@ -86,7 +85,7 @@ e_categories_master_list_nth_icon (ECategoriesMasterList *ecml, int n)
 	g_return_val_if_fail (ecml != NULL, NULL);
 	g_return_val_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml), NULL);
 
-	return ECML_CLASS (ecml)->nth_icon (ecml, n);
+	return E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->nth_icon (ecml, n);
 }
 
 /**
@@ -103,7 +102,7 @@ e_categories_master_list_nth_color (ECategoriesMasterList *ecml, int n)
 	g_return_val_if_fail (ecml != NULL, NULL);
 	g_return_val_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml), NULL);
 
-	return ECML_CLASS (ecml)->nth_color (ecml, n);
+	return E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->nth_color (ecml, n);
 }
 
 /**
@@ -123,8 +122,8 @@ e_categories_master_list_append (ECategoriesMasterList *ecml,
 	g_return_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml));
 	g_return_if_fail (category != NULL);
 
-	if (ECML_CLASS (ecml)->append)
-		ECML_CLASS (ecml)->append (ecml, category, color, icon);
+	if (E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->append)
+		E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->append (ecml, category, color, icon);
 }
 
 /**
@@ -138,8 +137,8 @@ e_categories_master_list_delete (ECategoriesMasterList *ecml, int n)
 	g_return_if_fail (ecml != NULL);
 	g_return_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml));
 
-	if (ECML_CLASS (ecml)->delete)
-		ECML_CLASS (ecml)->delete (ecml, n);
+	if (E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->delete)
+		E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->delete (ecml, n);
 }
 
 /**
@@ -154,7 +153,7 @@ e_categories_master_list_commit (ECategoriesMasterList *ecml)
 	g_return_if_fail (ecml != NULL);
 	g_return_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml));
 
-	ECML_CLASS (ecml)->commit (ecml);
+	E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->commit (ecml);
 }
 
 
@@ -170,25 +169,24 @@ e_categories_master_list_reset (ECategoriesMasterList *ecml)
 	g_return_if_fail (ecml != NULL);
 	g_return_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml));
 
-	ECML_CLASS (ecml)->reset (ecml);
+	E_CATEGORIES_MASTER_LIST_GET_CLASS (ecml)->reset (ecml);
 }
 
 
 static void
-e_categories_master_list_class_init (GtkObjectClass *object_class)
+e_categories_master_list_class_init (GObjectClass *object_class)
 {
 	ECategoriesMasterListClass *klass = E_CATEGORIES_MASTER_LIST_CLASS(object_class);
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 	
 	signals [CHANGED] =
-		gtk_signal_new ("changed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ECategoriesMasterListClass, changed),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, signals, LAST_SIGNAL);
+		g_signal_new ("changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ECategoriesMasterListClass, changed),
+			      NULL, NULL,
+			      e_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 
 	klass->count   = NULL;     
 	klass->nth     = NULL;         
@@ -202,30 +200,12 @@ e_categories_master_list_class_init (GtkObjectClass *object_class)
 }
 
 
-guint
-e_categories_master_list_get_type (void)
-{
-	static guint type = 0;
-	
-	if (!type)
-	{
-		GtkTypeInfo info =
-		{
-			"ECategoriesMasterList",
-			sizeof (ECategoriesMasterList),
-			sizeof (ECategoriesMasterListClass),
-			(GtkClassInitFunc) e_categories_master_list_class_init,
-			NULL,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-		
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
-
-  return type;
-}
+E_MAKE_TYPE (e_categories_master_list,
+	     "ECategoriesMasterList",
+	     ECategoriesMasterList,
+	     e_categories_master_list_class_init,
+	     NULL,
+	     PARENT_TYPE)
 
 /**
  * e_categories_master_list_changed:
@@ -241,6 +221,6 @@ e_categories_master_list_changed (ECategoriesMasterList *ecml)
 	g_return_if_fail (ecml != NULL);
 	g_return_if_fail (E_IS_CATEGORIES_MASTER_LIST (ecml));
 	
-	gtk_signal_emit (GTK_OBJECT (ecml),
-			 signals [CHANGED]);
+	g_signal_emit (ecml,
+		       signals [CHANGED], 0);
 }
