@@ -23,21 +23,22 @@
 
 #include <config.h>
 
-#include "e-reflow.h"
-
 #include <math.h>
-#include <gdk/gdkkeysyms.h>
-#include "e-canvas-utils.h"
-#include "e-canvas.h"
-#include "gal/e-text/e-text.h"
-#include "gal/util/e-util.h"
-#include "gal/util/e-i18n.h"
-#include "gal/widgets/e-unicode.h"
-#include <gtk/gtksignal.h>
-#include "e-selection-model-simple.h"
-
 #include <string.h>
 
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+
+#include "gal/e-text/e-text.h"
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+#include "gal/widgets/e-unicode.h"
+
+#include "e-canvas.h"
+#include "e-canvas-utils.h"
+#include "e-reflow.h"
+#include "e-selection-model-simple.h"
+
 static gboolean e_reflow_event (GnomeCanvasItem *item, GdkEvent *event);
 static void e_reflow_realize (GnomeCanvasItem *item);
 static void e_reflow_unrealize (GnomeCanvasItem *item);
@@ -168,8 +169,11 @@ do_adjustment (gpointer user_data)
 	gfloat value, min_value, max_value;
 	EReflow *reflow = user_data;
 
-	adj = gtk_layout_get_hadjustment (GTK_LAYOUT (GNOME_CANVAS_ITEM (reflow)->canvas));
 	row = reflow->cursor_row;
+	if (row == -1)
+		return FALSE;
+
+	adj = gtk_layout_get_hadjustment (GTK_LAYOUT (GNOME_CANVAS_ITEM (reflow)->canvas));
 	value = adj->value;
 
 	min_value = reflow->items[row]->x2 - adj->page_size;
