@@ -22,14 +22,16 @@
  */
 
 #include <config.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtktable.h>
-#include <libgnome/gnome-i18n.h>
-#include "gtk-combo-box.h"
-#include "widget-pixmap-combo.h"
+
+#include <libintl.h>		/* This file uses raw gettext() but no _() */
+
+#include <gtk/gtk.h>
+
 #include "gal/util/e-util.h"
 
+#include "gal-combo-box.h"
+#include "widget-pixmap-combo.h"
+
 #define PIXMAP_PREVIEW_WIDTH 15
 #define PIXMAP_PREVIEW_HEIGHT 15
 
@@ -38,7 +40,9 @@ enum {
 	LAST_SIGNAL
 };
 
-static gint pixmap_combo_signals [LAST_SIGNAL] = { 0, };
+static guint pixmap_combo_signals [LAST_SIGNAL] = { 0, };
+
+#define PARENT_TYPE GAL_COMBO_BOX_TYPE
 static GtkObjectClass *pixmap_combo_parent_class;
 
 /***************************************************************************/
@@ -49,7 +53,7 @@ pixmap_combo_destroy (GtkObject *object)
 	PixmapCombo *pc = PIXMAP_COMBO (object);
 
 	if (pc->tool_tip)
-		gtk_object_unref (GTK_OBJECT (pc->tool_tip));
+		g_object_unref (pc->tool_tip);
 	pc->tool_tip = NULL;
 	
 	(*pixmap_combo_parent_class->destroy) (object);
@@ -60,42 +64,24 @@ pixmap_combo_class_init (GtkObjectClass *object_class)
 {
 	object_class->destroy = pixmap_combo_destroy;
 
-	pixmap_combo_parent_class = gtk_type_class (gtk_combo_box_get_type ());
+	pixmap_combo_parent_class = g_type_class_ref (PARENT_TYPE);
 
 	pixmap_combo_signals [CHANGED] =
-		gtk_signal_new (
-			"changed",
-			GTK_RUN_LAST,
-			E_OBJECT_CLASS_TYPE (object_class),
-			GTK_SIGNAL_OFFSET (PixmapComboClass, changed),
-			gtk_marshal_NONE__INT,
-			GTK_TYPE_NONE, 1, GTK_TYPE_INT);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, pixmap_combo_signals, LAST_SIGNAL);
+		g_signal_new ("changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (PixmapComboClass, changed),
+			      NULL, NULL,
+			      e_marshal_NONE__INT,
+			      G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
-GtkType
-pixmap_combo_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"PixmapCombo",
-			sizeof (PixmapCombo),
-			sizeof (PixmapComboClass),
-			(GtkClassInitFunc) pixmap_combo_class_init,
-			(GtkObjectInitFunc) NULL,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (gtk_combo_box_get_type (), &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE (pixmap_combo,
+	     "PixmapCombo",
+	     PixmapCombo,
+	     pixmap_combo_class_init,
+	     NULL,
+	     PARENT_TYPE)
 
 static void
 emit_change (GtkWidget *button, PixmapCombo *pc)
@@ -104,8 +90,8 @@ emit_change (GtkWidget *button, PixmapCombo *pc)
 	g_return_if_fail (0 <= pc->last_index);
 	g_return_if_fail (pc->last_index < pc->num_elements);
 
-	gtk_signal_emit (GTK_OBJECT (pc), pixmap_combo_signals [CHANGED],
-			 pc->elements[pc->last_index].index);
+	g_signal_emit (pc, pixmap_combo_signals [CHANGED], 0,
+		       pc->elements[pc->last_index].index);
 }
 
 static void
@@ -114,7 +100,7 @@ pixmap_clicked (GtkWidget *button, PixmapCombo *pc)
 	int index = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (button)));
 	pixmap_combo_select_pixmap (pc, index);
 	emit_change (button, pc);
-	gtk_combo_box_popup_hide (GTK_COMBO_BOX (pc));
+	gal_combo_box_popup_hide (GAL_COMBO_BOX (pc));
 }
 
 static GtkWidget *
@@ -133,7 +119,7 @@ pixmap_table_setup (PixmapCombo *pc)
 
 	pc->combo_table = gtk_table_new (pc->cols, pc->rows, 0);
 	pc->tool_tip = gtk_tooltips_new ();
-	gtk_object_ref (GTK_OBJECT (pc->tool_tip));
+	g_object_ref (pc->tool_tip);
 	gtk_object_sink (GTK_OBJECT (pc->tool_tip));
 
 	for (row = 0; row < pc->rows; row++) {
@@ -160,8 +146,8 @@ pixmap_table_setup (PixmapCombo *pc)
 			gtk_table_attach (GTK_TABLE (pc->combo_table), button,
 					  col, col+1, row+1, row+2, GTK_FILL, GTK_FILL, 1, 1);
 
-			gtk_signal_connect (GTK_OBJECT (button), "clicked",
-					    GTK_SIGNAL_FUNC (pixmap_clicked), pc);
+			g_signal_connect (button, "clicked",
+					  G_CALLBACK (pixmap_clicked), pc);
 			gtk_object_set_user_data (GTK_OBJECT (button),
 						  GINT_TO_POINTER (index));
 		}
@@ -191,12 +177,12 @@ pixmap_combo_construct (PixmapCombo *pc,
 
 	gtk_container_add (GTK_CONTAINER (pc->preview_button), GTK_WIDGET (pc->preview_pixmap));
 	gtk_widget_set_usize (GTK_WIDGET (pc->preview_pixmap), 24, 24);
-	gtk_signal_connect (GTK_OBJECT (pc->preview_button), "clicked",
-			    GTK_SIGNAL_FUNC (emit_change), pc);
+	g_signal_connect (pc->preview_button, "clicked",
+			  G_CALLBACK (emit_change), pc);
 
 	gtk_widget_show_all (pc->preview_button);
 
-	gtk_combo_box_construct (GTK_COMBO_BOX (pc),
+	gal_combo_box_construct (GAL_COMBO_BOX (pc),
 				 pc->preview_button,
 				 pc->combo_table);
 }
@@ -211,7 +197,7 @@ pixmap_combo_new (PixmapComboElement const *elements, int ncols, int nrows)
 	g_return_val_if_fail (ncols > 0, NULL);
 	g_return_val_if_fail (nrows > 0, NULL);
 
-	pc = gtk_type_new (pixmap_combo_get_type ());
+	pc = g_object_new (PIXMAP_COMBO_TYPE, NULL);
 
 	pixmap_combo_construct (pc, elements, ncols, nrows);
 
